pub enum Coefficient<R: ?Sized + RingBase> {
Zero,
One,
NegOne,
Integer(i32),
Other(R::Element),
}Expand description
A coefficient used in a PlaintextCircuit.
Generally speaking, this always represents an element of R
(which can be retrieved via Coefficient::to_ring_el()), but
special cases are stored separately for a more efficient evaluation.
Variants
Implementations
Sourceimpl<R: ?Sized + RingBase> Coefficient<R>
impl<R: ?Sized + RingBase> Coefficient<R>
pub fn clone<S: RingStore<Type = R>>(&self, ring: S) -> Self
pub fn eq<S: RingStore<Type = R> + Copy>(&self, other: &Self, ring: S) -> bool
Sourcepub fn add_to<S: RingStore<Type = R> + Copy>(&self, x: El<S>, ring: S) -> El<S>
pub fn add_to<S: RingStore<Type = R> + Copy>(&self, x: El<S>, ring: S) -> El<S>
Computes self + x, but avoids a full ring addition if self is zero.
Sourcepub fn mul_to<S: RingStore<Type = R> + Copy>(&self, x: El<S>, ring: S) -> El<S>
pub fn mul_to<S: RingStore<Type = R> + Copy>(&self, x: El<S>, ring: S) -> El<S>
Computes self * x, but avoids a full ring multiplication if self
is stored as an integer.
pub fn is_zero(&self) -> bool
pub fn to_ring_el<S: RingStore<Type = R>>(self, ring: S) -> El<S>
pub fn negate<S: RingStore<Type = R>>(self, ring: S) -> Self
pub fn add<S: RingStore<Type = R> + Copy>(self, other: Self, ring: S) -> Self
pub fn mul<S: RingStore<Type = R> + Copy>(self, other: Self, ring: S) -> Self
Sourcepub fn change_ring<S, F>(self, f: F) -> Coefficient<S>
pub fn change_ring<S, F>(self, f: F) -> Coefficient<S>
Preserves integer coefficients, and maps ring elements as specified by the given function.