pub struct AlmostExactBaseConversion<A = Global>{ /* private fields */ }Expand description
Stores values for an almost exact conversion between RNS bases. A complete conversion refers to the function
Z/QZ -> Z/Q'Z, x -> [lift(x)]In our case, the output of the function is allowed to have an error of { -Q, 0, Q },
unless the shortest lift of the input is bounded by Q/4, in which case the result
is always correct.
Implementation
Implementation is changed to approximating the lifted value using lower precision integers, which can be used to determine the overflow when computing
lift(x) = sum_q lift(x * q/Q mod q) * Q/qmodulo some q'.
Implementations