pub struct AlmostExactMatrixBaseConversion<A = Global>{ /* private fields */ }Expand description
Stores values for an almost exact conversion between RNS bases. A complete conversion refers to the function
Z/QZ -> Z/Q'Z, x -> [lift(x)]In our case, the output of the function is allowed to have an error of { -Q, 0, Q },
unless the shortest lift of the input is bounded by Q/4, in which case the result
is always correct.
Implementation
Similar to super::lift::AlmostExactBaseConversion, but this
implementation makes some assumptions on the sizes of the moduli, which allows
to use a matrix multiplication for the performance-critical section.
Implementations